#!/usr/bin/env python
# -*- coding: utf-8 -*-

import numpy
import pfapack.pfaffian as pf

##################
# spin-1/2 chain #
##################

# chain length
N = 40

# <Sx(xi,ti)Sx(xj,0)>
xi = numpy.array([20, 21])
ti = numpy.array([-0.2, 0., 0.2, 0.4])
xj = numpy.array([20])
# note that the code is not threaded thus slow. Not a problem, because
# it is a script and we want fast solution rather than efficient
# coding at this point

# ----------------------------------------------------------

Jxy = 1. # energy scale
hz = 0.8 # external magnetic field
T = 0.002 # temperature
B = 1./T # 1/T

dk = numpy.pi/(N+1.);
k = dk*numpy.linspace(1,N,N); # numpy.linspace(dk,N*dk,N);

# ----------------------------------------------------------


# dispersion
E = lambda kn: Jxy*numpy.cos(kn) - hz;

AitAj = lambda i,t,j: (2./(N+1.))*( numpy.sum( \
        ( numpy.sin(k*j) * numpy.sin(k*i) ) * \
        ( +(1.+0.j) * numpy.cos(E(k)*t) - (0.+1.j) * numpy.sin(E(k)*t) * numpy.tanh(B*E(k)/2.) ) \
                  )     )
AitBj = lambda i,t,j: (2./(N+1.))*( numpy.sum( \
        ( numpy.sin(k*j) * numpy.sin(k*i) ) * \
        ( -(0.+1.j) * numpy.sin(E(k)*t) + (1.+0.j) * numpy.cos(E(k)*t) * numpy.tanh(B*E(k)/2.) ) \
                  )     )
BitBj = lambda i,t,j: -AitAj(i,t,j)
BitAj = lambda i,t,j: -AitBj(i,t,j)


#-----------------------------------------------------------

sxsxitj = numpy.zeros((len(xi),len(ti),len(xj)),dtype='complex128')

for tindex,time in enumerate(ti):
    for iindex,i in enumerate(xi.astype(int)):
        for jindex,j in enumerate(xj.astype(int)):
            # print("i,j",i,j)
            
            dim = 2*(i+j-1)
            PfafA = numpy.zeros((dim,dim),dtype='complex128')

            ### 2i-1 x 2i-2
            for matiindex in xrange(0,i):
                if matiindex != i-1:
                    for matjindex in xrange(matiindex,i-1):
                        # print('@[{0},1+{1}] : <A_{2}(0)B_{3}>'.format(2*matiindex,2*matjindex,matiindex+1,matjindex+1))
                        PfafA[2*matiindex,1+2*matjindex] = AitBj(matiindex+1,0,matjindex+1)
                for matjindex in xrange(matiindex+1,i):
                    # print('@[{0},1+{1}] : <A_{2}(0)A_{3}>'.format(2*matiindex,2*matjindex-1,matiindex+1,matjindex+1))
                    PfafA[2*matiindex,1+2*matjindex-1] = AitAj(matiindex+1,0,matjindex+1)            

            for matiindex in xrange(1,i):
                for matjindex in xrange(matiindex,i):
                    # print('@[{0},1+{1}] : <B_{2}(0)A_{3}>'.format(2*matiindex-1,2*matjindex-1,matiindex,matjindex+1))
                    PfafA[2*matiindex-1,1+2*matjindex-1] = BitAj(matiindex,0,matjindex+1)
                if matiindex != i-1:
                    for matjindex in xrange(matiindex,i-1):
                        # print('@[{0},1+{1}] : <B_{2}(0)B_{3}>'.format(2*matiindex-1,2*matjindex,matiindex,matjindex+1))
                        PfafA[2*matiindex-1,1+2*matjindex] = BitBj(matiindex,0,matjindex+1)
                        
            ### 2i-1 x 2j-1
            for matiindex in xrange(0,i):
                for matjindex in xrange(0,j):
                    # print('@[{0},1+2*i-2+{1}={2}] : <A_{3}(t)A_{4}>'.format(2*matiindex,2*matjindex,1+2*i-2+2*matjindex,matiindex+1,matjindex+1))
                    PfafA[2*matiindex,1+2*i-2+2*matjindex] = AitAj(matiindex+1,time,matjindex+1)
                if matiindex != i:
                    for matjindex in xrange(1,j):
                        # print('@[{0},1+2*i-2+{1}={2}] : <A_{3}(t)B_{4}>'.format(2*matiindex,2*matjindex-1,1+2*i-2+2*matjindex-1,matiindex+1,matjindex))
                        PfafA[2*matiindex,1+2*i-2+2*matjindex-1] = AitBj(matiindex+1,time,matjindex)
            for matiindex in xrange(1,i):
                for matjindex in xrange(0,j):
                    # print('@[{0},1+2*i-2+{1}={2}] : <B_{3}(t)A_{4}>'.format(2*matiindex-1,2*matjindex,1+2*i-2+2*matjindex,matiindex,matjindex+1))
                    PfafA[2*matiindex-1,1+2*i-2+2*matjindex] = BitAj(matiindex,time,matjindex+1)
                if matiindex != i:
                    for matjindex in xrange(1,j):
                        # print('@[{0},1+2*i-2+{1}={2}] : <B_{3}(t)B_{4}>'.format(2*matiindex-1,2*matjindex-1,1+2*i-2+2*matjindex-1,matiindex,matjindex))
                        PfafA[2*matiindex-1,1+2*i-2+2*matjindex-1] = BitBj(matiindex,time,matjindex)


            ### 2j-2 x 2j-1
            for matiindex in xrange(0,j-1):
                for matjindex in xrange(matiindex,j):
                    if matjindex!=j-1:
                        # print('@[2*i-1+{0}={1},1+2*i-1+{2}={3}] : <A_{4}(0)B_{5}>'.format(2*matiindex,2*i-1+2*matiindex,2*matjindex,1+2*i-1+2*matjindex,matiindex+1,matjindex+1))
                        PfafA[2*i-1+2*matiindex,1+2*i-1+2*matjindex] = AitBj(matiindex+1,0,matjindex+1)
                for matjindex in xrange(matiindex+1,j):
                    # print('@[2*i-1+{0}={1},1+2*i-1+{2}={3}] : <A_{4}(0)A_{5}>'.format(2*matiindex,2*i-1+2*matiindex,2*matjindex-1,1+2*i-1+2*matjindex-1,matiindex+1,matjindex+1))
                    PfafA[2*i-1+2*matiindex,1+2*i-1+2*matjindex-1] = AitAj(matiindex+1,0,matjindex+1)
            for matiindex in xrange(1,j):
                for matjindex in xrange(matiindex,j):
                    # print('@[2*i-1+{0}={1},1+2*i-1+{2}={3}] : <B_{4}(0)A_{5}>'.format(2*matiindex-1,2*i-1+2*matiindex-1,2*matjindex-1,1+2*i-1+2*matjindex-1,matiindex,matjindex+1))
                    PfafA[2*i-1+2*matiindex-1,1+2*i-1+2*matjindex-1] = BitAj(matiindex,0,matjindex+1)
                for matjindex in xrange(matiindex,j):
                    if matjindex!=j-1:
                        # print('@[2*i-1+{0}={1},1+2*i-1+{2}={3}] : <B_{4}(0)B_{5}>'.format(2*matiindex-1,2*i-1+2*matiindex-1,2*matjindex,1+2*i-1+2*matjindex,matiindex,matjindex+1))
                        PfafA[2*i-1+2*matiindex-1,1+2*i-1+2*matjindex] = BitBj(matiindex,0,matjindex+1)

            # sxsxitj[iindex,tindex,jindex] = numpy.sqrt(numpy.linalg.det(PfafA-numpy.transpose(PfafA)))/4. # sign problem ?!
            sxsxitj[iindex,tindex,jindex] = pf.pfaffian(PfafA-numpy.transpose(PfafA))/4.



# save to filename
filename = 'data/sxsx_N{N}_sh-xy_Jxy{Jxy}_hz{hz}_B{B}_szsz_xi{ximin}s{ximax}_ti{timin}s{timax}_xj{xjmin}s{xjmax}'.format(N=N,Jxy=Jxy,hz=hz,B=B,ximin=numpy.min(xi),ximax=numpy.max(xi),timin=numpy.min(ti),timax=numpy.max(ti),xjmin=numpy.min(xj),xjmax=numpy.max(xj))
numpy.savez(filename,N=N,Jxy=Jxy,hz=hz,B=B,xi=xi,ti=ti,xj=xj,sxsxitj=sxsxitj)
