#!/usr/bin/env python
# -*- coding: utf-8 -*-

import numpy

##################
# spin-1/2 chain #
##################

# chain length
N = 40

# <Sz(xi,ti)Sz(xj,0)>
xi = numpy.array([20,21])
ti = numpy.array([-0.2, 0., 0.2, 0.4])
xj = numpy.array([20])
# note that the code is not threaded thus slow. Not a problem, because
# it is a script and we want fast solution rather than efficient
# coding at this point

# ----------------------------------------------------------

Jxy = 1. # energy scale
hz = 0.8 # external magnetic field
T = 0.002 # temperature
B = 1./T # 1/T

dk = numpy.pi/(N+1.);
k = dk*numpy.linspace(1,N,N); # numpy.linspace(dk,N*dk,N);

# ----------------------------------------------------------


# dispersion
E = lambda kn: Jxy*numpy.cos(kn) - hz;


AitAj = lambda i,t,j: (2./(N+1.))*( numpy.sum( \
        ( numpy.sin(k*j) * numpy.sin(k*i) ) * \
        ( +(1.+0.j) * numpy.cos(E(k)*t) - (0.+1.j) * numpy.sin(E(k)*t) * numpy.tanh(B*E(k)/2.) ) \
                  )     )
AitBj = lambda i,t,j: (2./(N+1.))*( numpy.sum( \
        ( numpy.sin(k*j) * numpy.sin(k*i) ) * \
        ( -(0.+1.j) * numpy.sin(E(k)*t) + (1.+0.j) * numpy.cos(E(k)*t) * numpy.tanh(B*E(k)/2.) ) \
                  )     )
BitBj = lambda i,t,j: -AitAj(i,t,j)
BitAj = lambda i,t,j: -AitBj(i,t,j)


# ----------------------------------------------------------

szszitj   = numpy.zeros((len(xi),len(ti),len(xj)),dtype='complex128')
dszdszitj = numpy.zeros((len(xi),len(ti),len(xj)),dtype='complex128')

for tindex,time in enumerate(ti):
    for iindex,i in enumerate(xi):
        for jindex,j in enumerate(xj):
            szszitj[iindex,tindex,jindex]   =  ( AitBj(i,0,i)*AitBj(j,0,j)       -AitAj(i,time,j)*BitBj(i,time,j)+AitBj(i,time,j)*BitAj(i,time,j) )/4.
            dszdszitj[iindex,tindex,jindex]   =  (  -AitAj(i,time,j)*BitBj(i,time,j)+AitBj(i,time,j)*BitAj(i,time,j) )/4.
            

# magn            
xl=numpy.arange(1,N+1)
szl = (-1./(N+1.))*numpy.sum((numpy.sin(numpy.outer(xl,k))**2)*numpy.outer(numpy.ones(len(xl)),(numpy.tanh(B*E(k)/2.))),axis=1)
mz = numpy.sum(szl)/N
    
# save to filename
filename = 'data/szsz_N{N}_sh-xy_Jxy{Jxy}_hz{hz}_B{B}_szsz_xi{ximin}s{ximax}_ti{timin}s{timax}_xj{xjmin}s{xjmax}'.format(N=N,Jxy=Jxy,hz=hz,B=B,ximin=numpy.min(xi),ximax=numpy.max(xi),timin=numpy.min(ti),timax=numpy.max(ti),xjmin=numpy.min(xj),xjmax=numpy.max(xj))
numpy.savez(filename,N=N,Jxy=Jxy,hz=hz,B=B,xl=xl,szl=szl,mz=mz,xi=xi,ti=ti,xj=xj,szszitj=szszitj,dszdszitj=dszdszitj)
